--*********************************************************************************


--* This file is part of the RIBit Renderman(tm) exporter for max

--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************


try (DestroyDialog ro_RIBit_OutputParams)catch()


rollout ro_RIBit_OutputParams "RIBit 0.1 - Render Parameters"
(

global RIBit_Settings = s_RIBit_Settings()




-- Local Function Declerations
---------------------------------------
	local getCameraList




-- Local Variable Declerations
---------------------------------------
	local logFile = sysInfo.tempdir + "renderRIB.log"


-- User Interface
--------------------------------------
	group "Time Output " (
		radioButtons rbOutputTimeMode "" labels:#("Single", "Active Time Segment", "Range:", "Frames:") columns:1 align:#left pos:[12,22] enabled:false
		spinner spTimeRangeFrom "" fieldWidth:50 type:#integer range:[-1e8, 1e8, animationRange.start] align:#left pos:(rbOutputTimeMode.pos + [55,31]) enabled:false
		spinner spTimeRangeTo "To " fieldWidth:50 type:#integer range:[-1e8, 1e8, animationRange.end] align:#left pos:(spTimeRangeFrom.pos + [17,0]) enabled:false
		editText edTimeFrames "" fieldWidth:140 align:#left height:16 pos:(rbOutputTimeMode.pos + [54,48]) enabled:false
	)
	

	group "Output Size " (

		label lblFormatWidth "Width:" across:2 align:#left
		spinner spFormatWidth "" type:#integer range:[1,1e5,320] fieldWidth:47 align:#left offset:[-120,0]

		label lblFormatHeight "Height:" across:2 align:#left
		spinner spFormatHeight "" type:#integer range:[1,1e5,240] fieldWidth:47 align:#left offset:[-120,0]
		spinner spFormatPixelAspect "Pixel Aspect Ratio: " type:#float range:[0.0, 1000.0, 1.0] fieldWidth:47 align:#left
	)
	
	group "Options " (
		checkbox cbForceTwoSides "Force 2-Sided" checked:true
	)
	
	group "Main Display Settings " (
		editText edDispName "Display Name: " text:"unnamed" fieldWidth:200 across:2
		button bnDispNameBrowse "Browse..." height:16 width:50 offset:[60,0] enabled:false
		label lblDispType "Type: " align:#left across:3 offset:[0,5]
		dropDownList dlDispType "" items:#("Custom...", "framebuffer", "file", "tiff", "it") selection:2 align:#left offset:[-75,0]
		editText edDispTypeCustom "" filedWidth:100 offset:[-70,1] enabled:false
		editText edDispChanels "Channels:" text:"rgb" fieldWidth:50
	)
	
	group "Filter Settings " (
		label lblFilterType "Pixel Filter:" across:4 align:#left
		dropDownList dlFilterType "" items:#("box", "triangle", "catmull-rom", "gaussian", "sinc") selection:3 align:#left width:100 offset:[-30,-3]
		spinner spFilterWidthX "Filter Width:  X" fieldWidth:25 type:#float range:[0, 1e5, 2.0] align:#left offset:[5,0]

		spinner spFilterWidthY "Y" fieldWidth:25 type:#float range:[0, 1e5, 2.0] align:#left offset:[35,0]

		spinner spPixelSmalpesX "Pixel Samples:  X" fieldWidth:25 type:#float range:[0, 1e5, 2.0] align:#left across:3

		spinner spPixelSmalpesY "Y" fieldWidth:25 type:#float range:[0, 1e5, 2.0] align:#left offset:[15,0]


		spinner spPixelShadingRate "Shading Rate:" fieldWidth:35 type:#float range:[0, 1e5, 1.0] align:#left offset:[-9,0]

	)
	
	group "Rednderer Settings " (

		label lblRendererSelect "Select Renderer: " across:2 align:#left
		dropDownList dlRendererSelect "" width:100 items:#() align:#left offset:[-80,-4] selection:1
	)


	group "Camera Settings " (
		label lblCameraSelect "Select Camera: " across:3 align:#left
		dropDownList dlCameraSelect "" items:(getCameraList()) align:#left offset:[-20,-4] selection:2
		button bnRefreshList "Refresh List" height:18 offset:[-30,-4] 
	)
 
	group "Delete Files After Rendering" (
		label lblDelFilesKeep "Keep" across:3 align:#right offset:[80,0]
		label lblDelFilesKeepErr "Keep On Error" align:#right offset:[60,0]
		label lblDelFilesDel "Delete" align:#right offset:[0,0]
		
		label lblDelTempRIB "RIB Files: " align:#left across:2
		radioButtons rbDelTempRIB "" labels:#("             ", "              ", "") align:#left columns:3 offset:[10,0] default:3
		label lblDelTempShader "Compiled Shaders: " align:#left across:2
		radioButtons rbDelTempShader "" labels:#("             ", "              ", "") align:#left columns:3 offset:[10,0] default:1 enabled:false
	)


	group "Force Recomputing/Recomiling of: " (
		checkButton cbCacheShader "Shaders" width:75 height:18 align:#left checked:false highlightColor:(color 180 240 180)
		checkButton cbCacheShadows "Shadow Maps" width:75 height:18 align:#left pos:(cbCacheShader.pos + [80,0]) checked:true highlightColor:(color 180 240 180)
	)

	button bnViewLog "View Render Log" across:2 width:110 
	button bnRender "Render" width:80

-- Functions
-------------------------------
	fn getSceneCameraNames = for c in $cameras where (not isKindOf c Targetobject) collect c.name
	
	fn getCameraList = join #("Active Viewport") (getSceneCameraNames())



	fn setRenderSettings =
	(
		RIBit_Settings.rendWidth = spFormatWidth.value
		RIBit_Settings.rendHeight = spFormatHeight.value
		RIBit_Settings.rendPixelAspect = spFormatPixelAspect.value
		RIBit_Settings.filter = #(dlFilterType.selected, spFilterWidthX.value, spFilterWidthY.value)
		RIBit_Settings.pixelSamples = #(spPixelSmalpesX.value, spPixelSmalpesY.value)
		RIBit_Settings.shadingRate = spPixelShadingRate.value
		RIBit_Settings.dispName = if (getFilenamePath edDispName.text) == "" then (RIBit_Settings.ImageDir + edDispName.text) else edDispName.text
		RIBit_Settings.dispType = if dlDispType.selection > 1 then dlDispType.selected else edDispTypeCustom.text 
		RIBit_Settings.dispChannels = edDispChanels.text

		RIBit_Settings.renderOptions = #()
		if cbForceTwoSides.checked then
			append RIBit_Settings.renderOptions #force2sided 

		RIBit_Settings.activeRenderer = RIBit_Settings.renderers[dlRendererSelect.selection]

	) 

		
	
fn compileShaders shaders fileHandle logFile:undefined force:false =
	(

		if fileHandle == undefined then return()

		local ext = RIBit_Settings.renderers[dlRendererSelect.selection].compiledSLExt
		if ext == undefined then
			ext = ""


		if not force then
			shaders = for s in shaders where (not doesFileExist (s[2] + ext)) collect s

		if shaders.count == 0 then return()
		

		format "echo Rendering RIB Files...\n" to:fileHandle 
format "Compiling shaders.\n"
		
		local compilerApp = "\"" + RIBit_Settings.renderers[dlRendererSelect.selection].slCompiler + "\""
		local log = if logFile != undefined then (" >> " + logFile) else ""
		
		for s in shaders do (
			local shaderFile = getDir #scripts + "\\RIBit" + s[1]
			local params = "\"" + shaderFile + "\" -o \"" + s[2] + ext + "\"" + log
			if force or not doesFileExist (s[2] + ext) then
				format "% %\n" compilerApp params to:fileHandle
		)
		
	)
	

	
fn renderRIBs RIBFiles fileHandle logFile:undefined = 
	(
		if fileHandle == undefined then return()

	
	RIBFiles = for r in RIBFiles where (r != "" and r != undefined) collect r
	
	
	if RIBFiles.count < 1 then return 0

		format "echo Rendering RIB Files...\n" to:fileHandle

format "Rendering RIB files.\n"
	
		local rendererApp = s_RIBit_Functions.replaceBackslash RIBit_Settings.renderers[dlRendererSelect.selection].path
		local cmdLineParams = RIBit_Settings.renderers[dlRendererSelect.selection].cmdLineParams

		local log = ""
		if logFile != undefined then (
			log = (" >> " + logFile)
		)

		for r in RIBFiles do (

			format "\"%\" % \"%\"%\n" rendererApp cmdLineParams r log to:fileHandle
		)
	)
	
		
	fn generateShadowMaps frame force:true =
	(
format "Generating shadowmap RIB files.\n"
		setRenderSettings()
		local ribScene = s_RIBit_Scene settings:RIBit_Settings
		ribScene.frame = frame

		ribScene.type = #shadowmap
		ribScene.settings.rendPixelAspect = 1

		local fname = getFilenameFile MaxFilename
		if fname == "" then 
			fname = "Untitled"
		local frameStr = s_RIBit_Functions.leadingZero (s_RIBit_Functions.frameToInt frame) 4

		local lightList = for l in lights where (not isKindOf l Targetobject and l.enabled and l.baseObject.castShadows and not l.rayTracedShadows) collect l
		local shadowMaps = #()
		for l in lightList do (
			ribScene.renderCamera = l
			ribScene.settings.rendWidth = l.mapsize
			ribScene.settings.rendHeight = l.mapsize
			ribScene.settings.dispName = "SHD_" + l.name + "_" + fname + "_" + frameStr
			ribScene.settings.dispType = "zfile"
			ribScene.settings.dispChannels = "z"
			ribScene.settings.shadingRate = 2.0
	
		ribScene.settings = RIBit_Settings
			if force or not doesFileExist (ribScene.settings.textureDir + ribScene.settings.dispName + ".tx") then
				append shadowMaps (s_RIBit_Shadowmap l ribScene.settings.dispName (ribScene.exportScene()))
			else
				append shadowMaps (s_RIBit_Shadowmap l ribScene.settings.dispName "")
		)
		
		shadowMaps
	)
	
		
	fn deleteRIBFiles files =
	(
		for f in files do
			deleteFile f
	)
	
	
fn addBatchComment txt fileHandle =
	(		
		format "rem ============= % ================\n" txt to:fileHandle
	)
	
	
fn exportRIB =
	(
		local ribScene = s_RIBit_Scene()
		ribScene.frame = currentTime as integer
		ribScene.renderCamera = if (dlCameraSelect.selection > 1) then (getNodeByName dlCameraSelect.selected) else undefined

		
deleteFile logFile
		
		local batchFileName = sysInfo.tempdir + "RIBit_Render.bat"
		local batchFile = createFile batchFileName
		if batchFile == undefined then (
			format "Could not create batch file for rendering:\n  -->%\n" batchFileName
			return()
		)
		format "@ECHO OFF\n" to:batchFile
		

		ribScene.shadowmaps = generateShadowMaps ribScene.frame force:cbCacheShadows.state
		local RIBFiles = for s in ribScene.shadowmaps collect s.RIBFile

		addBatchComment "Begin Shadow Pass Renders" batchFile
		
renderRIBs RIBFiles batchFile

		addBatchComment "End Shadow Pass Renders" batchFile
		

		setRenderSettings()
		ribScene.settings = RIBit_Settings
		
		local RIBFile = ribScene.exportScene()
		addBatchComment "Begin Shaders Compiling" batchFile
		compileShaders ribScene.shadersToCompile batchFile logFile:logFile force:cbCacheShader.state
		addBatchComment "End Shaders Compiling" batchFile
		addBatchComment "Begin Image Render" batchFile
		renderRIBs #(RIBFile) batchFile logFile:logFile

		addBatchComment "End Image Render" batchFile

		
local rendResult = dosCommand batchFileName
		
		join RIBFiles #(RIBFile)
		
case rbDelTempRIB.state of (
			2: if rendResult == 0 then deleteRIBFiles RIBFiles
			3: deleteRIBFiles RIBFiles
		)
	)
	
	
fn updateUI =
	(
		edDispTypeCustom.enabled = dlDispType.selection == 1
	)
	
	
	
fn init =
	(
		RIBit_Settings.readFromFile()
		dlRendererSelect.items = for i in RIBit_Settings.renderers collect i.name
	)
	
	
-- Event Handlers
-------------------------------

	on dlDispType selected val do (

		updateUI()
	)

	on 

bnRefreshList pressed do (
		dlCameraSelect.items = getCameraList()
	)

	on bnRender pressed do exportRIB()

	on bnViewLog pressed do edit logFile

	on ro_RIBit_OutputParams open do init()

) -- end of ro_RIBit_OutputParams rollout

createDialog ro_RIBit_OutputParams width:350 style:#(#style_border, #style_titlebar, #style_sysmenu, #style_minimizebox)