--*********************************************************************************


--* This file is part of the RIBit Renderman(tm) exporter for max

--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************


struct s_RIBit_Camera (
	cam,
	settings = s_RIBit_Settings(),
	indent = "",
	fileHandle,
	
	

	fn exportRIBBox =
	(
		if isValidNode cam then (
			local modIndex = s_RIBit_Functions.findModifier cam mo_RIBit_RIBBox multiple:true
			if modIndex != undefined then (
				format "%##---- RIB Box ----##\n" indent to:fileHandle
				for i in modIndex do (
					local lines = filterString (s_RIBit_Functions.parseRIBBox cam.modifiers[i].RIBCode) "\n\r"
					for l in lines do
						format "%%\n" indent l to:fileHandle
				)
			)

		)
	),



	fn expotLightAsCamera =
	(

		local cameraVFOV = if (isKindOf cam freeSpot or isKindOf cam targetSpot) then cam.falloff else 95
		local nearClip = 0.1
		local farClip = 100000.0
		if isKindOf cam TargetDirectionallight or isKindOf cam Directionallight then (
			Format "%Projection \"orthographic\"\n" indent to:fileHandle
			local xmin = -cam.falloff
			local xmax = cam.falloff
			local ymin = -(cam.falloff / cam.aspect)
			local ymax = (cam.falloff / cam.aspect)
			Format "%ScreenWindow % % % %\n" indent xmin xmax ymin ymax to:fileHandle
		) else (

		Format "%Projection \"perspective\" \"fov\" %\n" indent cameraVFOV to:fileHandle
		)
		Format "%Clipping % %\n" indent nearClip farClip to:fileHandle

		local ribTransform = s_RIBit_Transform (cam.transform) indent:indent fileHandle:fileHandle
		ribTransform.inversePOV()
		ribTransform.write()
	),



	fn exportCamera =
	(
		local renderAspect = (settings.rendWidth as float)/settings.rendHeight
		local cameraVFOV = if renderAspect > 1 then (2.*atan(tan(cam.fov/2.)/renderAspect)) else cam.fov
		local ribTransform = s_RIBit_Transform (cam.transform) indent:indent fileHandle:fileHandle

		if cam.orthoProjection then (
			Format "%Projection \"orthographic\"\n" indent to:fileHandle
			ribTransform.inversePOV()
		) else (
			Format "%Projection \"perspective\" \"fov\" %\n" indent cameraVFOV to:fileHandle
			ribTransform.inversePOV()
		)

		if cam.clipManually then
			Format "%Clipping % %\n" indent cam.nearClip cam.farClip to:fileHandle

		ribTransform.write()
	
	),
	
	
	fn exportViewport =
	(
		local viewtype = viewport.getType()

		if (viewtype == #view_camera) then (
			cam = viewport.getCamera()
			exportCamera()			
		) else if (viewtype == #view_persp_user) then (
		--- code from doberman
			local renderAspect = (settings.rendWidth as float)/settings.rendHeight
			local cameraVFOV = if renderAspect > 1 then (2.*atan(tan(getViewFOV()/2.)/renderAspect)) else getViewFOV()
			local nearClip = 0.1
			local farClip = 100000.0
			if cam != undefined then
				if cam.clipManually == true then
				(
					local nearClip = cam.nearclip
					local farClip = cam.farclip
				)
			Format "%Projection \"perspective\" \"fov\" %\n" indent cameraVFOV to:fileHandle
			Format "%Clipping % %\n" indent nearClip farClip to:fileHandle
			
			local ribTransform = s_RIBit_Transform (inverse (getViewTM())) indent:indent fileHandle:fileHandle
			ribTransform.inversePOV()
			ribTransform.write()

		) else (
--				format "%Projection \"orthographic\"\n" indent to:fileHandle
--				local r = mapScreenToWorldRay (getViewSize() / 2.)
--				local ribTransform = s_RIBit_Transform (matrixFromNormal r.dir) indent:indent fileHandle:fileHandle
--				ribTransform.inversePOV()
--		transform = inverse transform
--		ribTransform.transform = translate ribTransform.transform r.pos
--		ribTransform.transform = inverse ribTransform.transform 
--		ribTransform.inversePOV()
--		scale [-1,1,1]
--				ribTransform.write()

			) 
	),
	


	fn write =
	(
		if fileHandle == undefined then return()
		
		
case (superClassOf cam) of (
			-- if cam is a light, we're exporting a shadowmap RIB. 
			light: expotLightAsCamera()
			-- if cam is a camera object, use it.
			camera: exportCamera()
			-- otherwise, export the active viewport.
			default: exportViewport()
		)
		exportRIBBox()


	
)

) -- end of s_RIBit_Camera struct


