--*********************************************************************************


--* This file is part of the RIBit Renderman(tm) exporter for max

--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************

struct s_RIBit_Geometry (
	object,

	force2sided = false,
	indent = "",
	fileHandle,
	
	
	fn exportPolygons type =
	(
		max create mode
		disableSceneRedraw()
		------------------
		-- check whether it is a poly
		-----------------
		local obj = snapshot object
		convertToPoly obj
		local facecount = getNumFaces obj
		
		case type of (
			1: format "%PointsPolygons \n" indent to:fileHandle
			2: format "%SubdivisionMesh \"catmull-clark\"\n" indent to:fileHandle
		)
		
		--if ((mode == 1) or (mode == 2)) then(
		--First output the number of vertices per face
		format "%\t[" indent to:fileHandle
		for i=1 to facecount do(
			if (mod i 25 == 0 )then
				format"\n%\t" indent to:fileHandle
			local faceverts = polyop.getFaceVerts obj i
			local numverts = faceverts.count
			format " % " numverts to:fileHandle
		)
		format "]\n" to:fileHandle
		
		-- now output the vertex indexes
		format "%\t[" indent to:fileHandle
		for i=1 to facecount do(
			if (mod i 9 == 0 ) then
				format"\n%\t" indent to:fileHandle
			local faceverts = polyop.getFaceVerts obj i
			for j = 1 to faceverts.count do(
				format " % " (faceverts[j]-1) to:fileHandle
			)
		)
		format "]\n" to:fileHandle
		
		-- now output the position of each vertex
		format "%\t\"P\" [" indent to:fileHandle
		local numverts = getNumVerts obj
		for i = 1 to numverts do (
			if (mod i 9 == 0 )then
				format"\n%\t" indent to:fileHandle
			vertpos = in coordsys local polyop.getVert obj i
			format " % % % " vertpos[1] vertpos[2] vertpos[3] to:fileHandle
		)
		format "]\n" to:fileHandle

		-- now output the normal of each vertex
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Geometry
		if modIndex == undefined or object.modifiers[modIndex].ExportNormals then (
			format "%\t\"N\" [" indent to:fileHandle
			convertToMesh obj
			local numverts = getNumVerts obj
			for i = 1 to numverts do (
				if (mod i 9 == 0 )then
					format"\n%\t" indent to:fileHandle
				normpos = in coordsys world getNormal obj i
				format " % % % " normpos[1] normpos[2] normpos[3] to:fileHandle
			)
			format "]\n" to:fileHandle
		)
	
		delete obj
		enablesceneredraw()
	),


	fn exportRenderingAttributes =
	(
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Shaders
		if modIndex == undefined then (
			local objcolor = object.wirecolor/255.
			format "%Color % % %\n" indent objcolor.r objcolor.g objcolor.b to:fileHandle
			format "%Opacity % % %\n" indent objcolor.a objcolor.a objcolor.a to:fileHandle
			format "%Surface \"matte\"\n" indent to:fileHandle
		) else (
			objcolor = object.modifiers[modIndex].Color/255.
			format "%Color [% % % ]\n" indent objcolor.r objcolor.g objcolor.b to:fileHandle
			objOpac = object.modifiers[modIndex].Opacity/255
			format "%Opacity [% % %]\n" indent objOpac.r objOpac.g objOpac.b to:fileHandle
			format "%Surface \"%\"\n" indent object.modifiers[modIndex].Surface to:fileHandle
			if object.modifiers[modIndex].Displacement != "" then (
				format "%Displacement \"%\"\n" indent object.modifiers[modIndex].Displacement to:fileHandle
			)
			if object.modifiers[modIndex].Atmosphere != "" then(
				format "%Atmosphere \"%\"\n" indent object.modifiers[modIndex].Atmosphere to:fileHandle
			)
			format "%ShadingInterpolation \"%\"\n" indent object.modifiers[modIndex].ShadingInterpolation to:fileHandle
		)
	),

	fn exportRIBBox =
	(
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_RIBBox multiple:true
		if modIndex != undefined then (
			format "%##---- RIB Box ----##\n" indent to:fileHandle
			for i in modIndex do (
				local lines = filterString (s_RIBit_Functions.parseRIBBox object.modifiers[i].RIBCode) "\n\r"
				for l in lines do
					format "%%\n" indent l to:fileHandle
			)
		)
	),

	fn write =
	(
		if fileHandle == undefined then return()
		if (isKindOf object GeometryClass) and not (isKindOf object targetObject) and object.renderable and not object.isHidden then (
			-- Rendering Attributes
			exportRenderingAttributes()



			local ribTransform = s_RIBit_Transform object.transform concat:true indent:indent fileHandle:fileHandle
			ribTransform.transformBegin()
			indent = ribTransform.indent
			ribTransform.write()

			exportRIBBox()

			-- Export Polygons
			local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Geometry
			if modIndex == undefined then 
				exportPolygons 1
			else (
				if (object.modifiers[modIndex].ExportPoly and not object.modifiers[modIndex].ExportSubd) then
					exportPolygons 1
				if (object.modifiers[modIndex].ExportPoly and object.modifiers[modIndex].ExportSubd) then
					exportPolygons 2
			)

			ribTransform.transformEnd()
			indent = ribTransform.indent
			-- End Transformation block
		) 
	)

) -- end of s_RIBit_Geometry struct













