--*********************************************************************************


--* This file is part of the RIBit Renderman(tm) exporter for max

--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************

struct s_RIBit_Scene (
	frame = 0,
	type = #scene,
	RIBFilename,
	renderCamera,
	settings = s_RIBit_Settings(),
	shadersToCompile = #(),
	shadowmaps = #(),
	indent = "",
	fileHandle,
	
	
fn incIndent = 
	(
		indent += "\t"
	),
	
	fn decIndent =
	(
		if indent.count > 0 then
			indent = subString indent 1 (indent.count - 1)
	),
	
	
fn writeComment comment indent:indent =
	(

		if fileHandle == undefined then return()
		comment = comment as string
		format "%#%\n" indent comment to:fileHandle 
	),



	fn writeHeader = 
	(
		writeComment "# RIB file structure"
		writeComment "# Generated using RIBit V 0.1 by Ofer Zelichover"
		writeComment "# Base on kcrib by Rudi Cortes"
		writeComment ("# Created on " + localtime + " by " + sysInfo.username + " on 3DSMax ver. " \
						+ (maxversion() as string) + "\n")
	),


	fn writeSearchPaths =
	(
		format "%Option \"searchpath\" \"shader\" \"%:&\"\n" indent (s_RIBit_Functions.replaceBackslash settings.ShaderDir) to:fileHandle
		format "%Option \"searchpath\" \"texture\" \"%:&\"\n" indent (s_RIBit_Functions.replaceBackslash settings.TextureDir) to:fileHandle
		format "\n" to:fileHandle
	),


	fn writeDisplay =
	(
		local n = settings.dispName
		if type == #shadowmap then
			n += ".depth"
		local ribDisplay= s_RIBit_Display name:n type:settings.dispType channels:settings.dispChannels fileHandle:fileHandle indent:indent
		ribDisplay.write()
	),
	
	fn writeFormat =
	(
		local ribFormat = s_RIBit_Option "Format" fileHandle:fileHandle indent:indent
		ribFormat.addParameter settings.rendWidth undefined
		ribFormat.addParameter settings.rendHeight undefined

		ribFormat.addParameter settings.rendPixelAspect undefined

		ribFormat.write()
	),
	
	
fn writeFilter =
	(
		local ribFilter = s_RIBit_Option "PixelFilter" fileHandle:fileHandle indent:indent
		ribFilter.addParameter settings.filter[1] undefined
		ribFilter.addParameter settings.filter[2] undefined
		ribFilter.addParameter settings.filter[3] undefined
		ribFilter.write()
		
		
local ribPixelSamples = s_RIBit_Option "PixelSamples" fileHandle:fileHandle indent:indent
		ribPixelSamples.addParameter settings.pixelSamples[1] undefined
		ribPixelSamples.addParameter settings.pixelSamples[2] undefined
		ribPixelSamples.write()
		
		local ribShadingRate = s_RIBit_Option "ShadingRate" fileHandle:fileHandle indent:indent
		ribShadingRate.addParameter settings.shadingRate undefined
		ribShadingRate.write()
	),
	

	fn writeBackground =
	(
		if backgroundColor != black then (
			local ribImager = s_RIBit_Option "Imager" fileHandle:fileHandle indent:indent
			ribImager.addParameter "background" undefined
			ribImager.addParameter "color bgcolor" #((backgroundColor.r/255), (backgroundColor.g/255), (backgroundColor.b/255))
			ribImager.write()
		)
	),


	fn writeRenderOptions =
	(
		if findItem settings.RenderOptions #force2sided > 0 then (
			local ribOption = s_RIBit_Option "Sides" fileHandle:fileHandle indent:indent
			ribOption.addParameter 2 undefined
			ribOption.write()
		)
		
	),


	fn exportLights =
	(
		writeComment "# Light Declarations"

	
	-- export ambient light, if present.
		if ambientColor != black then (
			local ribLight = s_RIBit_Light ambientColor lightID:0 shadersDir:settings.ShaderDir indent:indent fileHandle:fileHandle
			ribLight.write()
		)			
	
		local allLights = for l in lights where (not isKindOf l Targetobject) collect l
		if allLights.count == 0 then 
			writeComment "# No Lights on this scene\n"
		else (
			local shdmapLights = for o in shadowmaps collect o.object
			
for i = 1 to allLights.count do (
				local ribLight = s_RIBit_Light allLights[i] lightID:i shadersDir:settings.ShaderDir shaderExt:settings.activeRenderer.compiledSLExt \
										 shadowSamples:(32 / settings.shadingRate) indent:indent fileHandle:fileHandle
				local shdmapIndex = findItem shdmapLights allLights[i]
				if shdmapIndex > 0 then
					ribLight.shadowmapName = shadowmaps[shdmapIndex].name + ".tx"
				ribLight.write()

				s_RIBit_Functions.uniqueJoin shadersToCompile ribLight.shadersToComplie
			)
		)
	),

	
	fn exportObjects obj =
	(
		if obj == undefined then return()
		if isKindOf obj GeometryClass and not (isKindOf obj targetObject) and obj.renderable and not obj.isHidden then (

			local ribObject = s_RIBit_Geometry obj indent:(indent + "\t") fileHandle:fileHandle
			ribObject.force2sided = (findItem settings.renderOptions #force2sided) > 0
			writeComment ("#---- " + obj.name + " decalration starts ----##")
			format "%AttributeBegin\n" indent to:fileHandle
			
			ribObject.write()
		)
		incIndent()
		for o in obj.children do 
			exportObjects o
		decIndent()

		if isKindOf obj GeometryClass and not (isKindOf obj targetObject) and obj.renderable and not obj.isHidden then (


			format "%AttributeEnd\n" indent to:fileHandle
			writeComment ("#---- " + obj.name + " decalration ends ----##")
		)

	),
	
	
fn genRIBFileName =
	(
		local RIBType = ""
		case type of (
			#shadowmap: RIBType = "SHD_" + renderCamera.name + "_"
		)

		local fname = getFilenameFile MaxFilename
		if fname == "" then 
			fname = "Untitled"
		local frameStr = s_RIBit_Functions.leadingZero (s_RIBit_Functions.frameToInt frame) 4

		RIBFilename = settings.RIBDir + RIBType + fname + "_" + frameStr + ".rib"
	),
	
	fn exportScene =
	(
		if RIBFilename == undefined or RIBFilename == "" or type != #scene then
			genRIBFileName()
	
	fileHandle = createFile RIBFilename

		if fileHandle == undefined then return()

		-- Write some general info
		writeHeader()
		writeSearchPaths()
		

		local ribFrame = s_RIBit_Frame frame:frame fileHandle:fileHandle
		ribFrame.FrameBegin()
		incIndent()

		
		-- Export Display information
		writeComment "# Image Display Declaration" to:fileHandle 

		writeDisplay()

		writeFormat()

		if type == #scene then (
			writeFilter()

			writeBackground()
		)
		writeRenderOptions()


		-- Export Camera Information
		local ribCamera = s_RIBit_Camera cam:renderCamera settings:settings fileHandle:fileHandle indent:indent
		ribCamera.write()

		-- World Begin
		local ribWorld = s_RIBit_World indent:indent fileHandle:fileHandle
		ribWorld.WorldBegin()

		incIndent()
		if type == #scene then
			exportLights()
		decIndent() 
		exportObjects rootNode

		ribWorld.WorldEnd()
		decIndent()
		

		if type == #shadowmap then (
			local shadowName = s_RIBit_Functions.replaceBackSlash (settings.textureDir + settings.dispName + ".tx")
			format "%MakeShadow \"%\" \"%\"\n" indent (settings.dispName + ".depth") shadowName to:fileHandle
		)
		
		ribFrame.FrameEnd()
		
		close fileHandle
		
		return RIBFilename
	)
	


) -- end of s_RIBit_Scene

